@@ -29,6 +29,9 @@
 /* Various includes */
 #include <libxml/HTMLparser.h>
 #include <libxml/tree.h>
+#include <libxml/xpath.h>
+#include <pango/pangox.h>
+#include <langinfo.h>
 
 /* Platform specific includes */
 #include <X11/X.h>
@@ -55,30 +58,19 @@
 
 #include "../config.h"
 
-#define SEARCH_METHOD_FORWARD        0
-#define SEARCH_METHOD_BACKWARD       1
-#define SEARCH_METHOD_EXACTWORD      2
-#define SEARCH_METHOD_KEYWORD        3
-#define SEARCH_METHOD_MENU          10
-#define SEARCH_METHOD_COPYRIGHT     11
-#define SEARCH_METHOD_UNKNOWN       99
-
-#define SEARCH_FORWARD      _("Forward")
-#define SEARCH_BACKWARD     _("Backward")
-#define SEARCH_EXACTWORD    _("Exactword")
-#define SEARCH_KEYWORD      _("Keyword")
-#define SEARCH_MENU         _("Menu")
-#define SEARCH_COPYRIGHT    _("Copyright")
-
 #define ENC_UTF8            "utf-8"
 #define ENC_EUC_JP          "euc-jp"
 #define ENC_SHIFT_JIS       "shift-jis"
 #define ENC_ISO_2022_JP     "iso-2022-jp"
 #define ENC_ASCII           "ascii"
 
+#ifndef SZ
+#define SZ(a) (sizeof(a)/sizeof(a[0]))
+#endif
+
 typedef struct
 {
-    gboolean active, available;
+    gboolean active;
     gchar *title, *path, *appendix_path, *subbook_dir;
     EB_Book *book;
     EB_Appendix *appendix;
@@ -87,6 +79,7 @@ typedef struct
 
 typedef struct _result_eb
 {
+    gchar *book_title;
     gchar *heading;
     BOOK_INFO *binfo;
     EB_Position pos;
@@ -94,12 +87,15 @@ typedef struct _result_eb
     gchar *filename;
 } RESULT;
 
-struct _search_method
+enum
 {
-    gint  code;
-    gchar *name;
-    int (* have_method)(EB_Book *book);
-    EB_Error_Code (* search)(EB_Book *book, const gchar* input_word);
+    SEARCH_METHOD_FORWARD,
+    SEARCH_METHOD_BACKWARD,
+    SEARCH_METHOD_EXACTWORD,
+    SEARCH_METHOD_KEYWORD,
+    SEARCH_METHOD_MENU,
+    SEARCH_METHOD_COPYRIGHT,
+    SEARCH_METHOD_UNKNOWN
 };
 
 enum
@@ -118,8 +114,12 @@ enum
     DICT_N
 };
 
-
 #include "log.h"
 
+#define FILENAME_DICTS	    "dictionaries.xml"
+#define FILENAME_PREFS	    "preferences.xml"
+#define FILENAME_HOTKEYS    "hotkeys.xml"
+#define FILENAME_HISTORY    "history.xml"
+
 #endif /* __DEFS_H__ */
 
