@@ -1,47 +1,18 @@
 
-#define _GLOBAL
-#include <sys/wait.h>
-
 #include "defs.h"
 
-#include "dictbar.h"
+#include <locale.h>
 #include "dicts.h"
-#include "headword.h"
 #include "history.h"
 #include "hook.h"
 #include "mainwnd.h"
-#include "preferences.h"
-#include "selection.h"
+#include "prefs.h"
 #include "hotkeys.h"
-#include "textview.h"
 
 #ifdef ENABLE_GSTREAMER
 #include <gst/gst.h>
 #endif
 
-void app_exit(GtkWidget *widget, gpointer   data)
-{
-    hooksets_finalize();
-
-    gdk_window_get_root_origin(mainwnd.wnd->window, &mainwnd.x, &mainwnd.y);
-    mainwnd.w = mainwnd.wnd->allocation.width;
-    mainwnd.h = mainwnd.wnd->allocation.height;
-
-    paned.treew = paned.tree->allocation.width;
-    paned.treeh = paned.tree->allocation.height;
-
-    history_save_words();
-    preferences_save();
-    dicts_save();
-    hotkeys_save();
-    gtk_main_quit ();
-}
-
-void app_quit()
-{
-    app_exit(NULL, NULL);
-}
-
 gchar* app_browse_disk(gchar* title, GtkWindow *wnd, gint action, gchar *btn_text, gchar *path)
 {
     gchar *res = NULL;
@@ -61,26 +32,22 @@ int main(int argc, char *argv[])
 {
     gtk_init(&argc, &argv);
 
-#ifdef ENABLE_GSTREAMER
-    gst_init(&argc, &argv);
-#endif
-
 #ifdef ENABLE_NLS
     bindtextdomain(PACKAGE_NAME, LOCALEDIR);
     bind_textdomain_codeset(PACKAGE_NAME, ENC_UTF8);
     textdomain(PACKAGE_NAME);
 #endif
 
-    preferences_init();
+#ifdef ENABLE_GSTREAMER
+    gst_init(&argc, &argv);
+#endif
+
     eb_initialize_library();
     hooksets_init();
 
-    preferences_load();
-    dicts_load();
-    history_load_words();
-    hotkeys_load();
-
-    mainwnd_create();
+    Mainwnd *mainwnd = g_object_new(TYPE_MAINWND, NULL);
+    mainwnd_prepare(mainwnd);
+    gtk_widget_show_all(GTK_WIDGET(mainwnd));
 
     gtk_main();
     hooksets_finalize();
