@@ -1,11 +1,9 @@
 
-#ifndef __GSTEBSRC_H__
-#define __GSTEBSRC_H__
+#ifndef __EB_GSTSRC_H__
+#define __EB_GSTSRC_H__
 
 #ifdef ENABLE_GSTREAMER
 
-#include "defs.h"
-
 G_BEGIN_DECLS
 
 #define EB_GST_TYPE_SRC                  (eb_gst_src_get_type())
@@ -37,5 +35,5 @@ GType eb_gst_src_get_type(void);
 
 G_END_DECLS
 
-#endif
-#endif
+#endif /* GSTREAMER */
+#endif /* __EB_GSTSRC_H__ */
