@@ -1,10 +1,9 @@
 
 #include "defs.h"
 
-#include "dictbar.h"
+#include "builder.h"
 #include "dicts.h"
 #include "ebook.h"
-#include "headword.h"
 #include "history.h"
 #include "hook.h"
 #include "jcode.h"
@@ -15,104 +14,121 @@
 #define MAX_BUFSIZE 65535
 #endif
 
-struct _search_method methods[] = {
-    {SEARCH_METHOD_FORWARD,     SEARCH_FORWARD,	    eb_have_word_search,	eb_search_word},
-    {SEARCH_METHOD_BACKWARD,    SEARCH_BACKWARD,    eb_have_endword_search,	eb_search_endword},
-    {SEARCH_METHOD_EXACTWORD,   SEARCH_EXACTWORD,   eb_have_exactword_search,	eb_search_exactword},
-    {SEARCH_METHOD_KEYWORD,     SEARCH_KEYWORD,	    eb_have_keyword_search,	ebook_search_keyword},
-    {SEARCH_METHOD_MENU,	SEARCH_MENU,	    eb_have_menu,		NULL},
-    {SEARCH_METHOD_COPYRIGHT,	SEARCH_COPYRIGHT,   eb_have_copyright,		NULL},
-    {0, NULL, NULL, NULL}
+typedef struct
+{
+    gchar *name;
+    int (* have_method)(EB_Book *book);
+    EB_Error_Code (* search)(EB_Book *book, const gchar* input_word);
+} SEARCH_METHOD;
+
+struct _ebook
+{
+    GSequence   *results;
+} ebook;
+
+static const SEARCH_METHOD methods[] =
+{
+    {_("Forward"),	    eb_have_word_search,	eb_search_word},
+    {_("Backward"),	    eb_have_endword_search,	eb_search_endword},
+    {_("Exactword"),	    eb_have_exactword_search,	eb_search_exactword},
+    {_("Keyword"),	    eb_have_keyword_search,	ebook_search_keyword},
+    {_("Menu"),		    eb_have_menu,		NULL},
+    {_("Copyright"),	    eb_have_copyright,		NULL},
 };
 
-gint ebook_search_method()
+const gchar* ebook_index_to_method_name(gint index)
 {
-    gint i;
-    const gchar *method_name = gtk_combo_box_get_active_text(GTK_COMBO_BOX(mainwnd.combo_method));
-    for(i = 0; methods[i].name != NULL; i++)
-        if(!g_strcmp0(method_name, methods[i].name))
-            return methods[i].code;
-    return SEARCH_METHOD_UNKNOWN;
+    if(index < 0)
+	return NULL;
+    if(index >= SZ(methods))
+	return NULL;
+    SEARCH_METHOD m = methods[index];
+    return m.name;
 }
 
-BOOK_INFO *ebook_load(const gchar *path, int subbook_no)
+gboolean ebook_has_method(EB_Book *book, gint index)
+{
+    if(index < 0)
+	return False;
+    if(index >= SZ(methods))
+	return False;
+    return methods[index].have_method(book);
+}
+
+gboolean ebook_load(BOOK_INFO *binfo)
 {
     gint subcount;
     gchar buff[512];
-    BOOK_INFO *binfo;
     EB_Error_Code error_code;
     EB_Subbook_Code sublist[EB_MAX_SUBBOOKS];
-
-    if(!path)
-        return NULL;
-
-    binfo = (BOOK_INFO*)g_new0(BOOK_INFO, 1);
     if(!binfo)
-        return NULL;
+	return False;
+    if(!binfo->path)
+        return False;
 
-    binfo->path = fs_to_unicode((gchar*)path);
-    binfo->subbook_no = subbook_no;
-
-    binfo->book = (EB_Book*)g_new0(EB_Book, 1);
+    binfo->book = (EB_Book*)g_malloc0(sizeof(EB_Book) + 5);
     if(!binfo->book)
-    {
-        ebook_free(binfo);
-        return NULL;
-    }
+        return False;
+
     eb_initialize_book(binfo->book);
-    error_code = eb_bind(binfo->book, path);
+    error_code = eb_bind(binfo->book, binfo->path);
     if(error_code != EB_SUCCESS)
     {
-        ebook_free(binfo);
-        LOG(LOG_CRITICAL, "Failed to bind the book %s: %s", path, eb_error_message(error_code));
-        return NULL;
+        g_free(binfo->book);
+	binfo->book = NULL;
+        LOG(LOG_CRITICAL, "Failed to bind the book %s: %s", binfo->path, eb_error_message(error_code));
+        return False;
     }
     error_code = eb_subbook_list(binfo->book, sublist, &subcount);
     if(error_code != EB_SUCCESS)
     {
-        ebook_free(binfo);
+        g_free(binfo->book);
+	binfo->book = NULL;
         LOG(LOG_CRITICAL, "Failed to get a subbook list: %s", eb_error_message(error_code));
-        return NULL;
+        return False;
     }
-    if(subbook_no >= subcount)
+    if(binfo->subbook_no >= subcount)
     {
-        ebook_free(binfo);
-        return NULL;
+        g_free(binfo->book);
+	binfo->book = NULL;
+        return False;
     }
     error_code = eb_subbook_directory2(binfo->book, sublist[binfo->subbook_no], buff);
     if (error_code != EB_SUCCESS)
     {
-        ebook_free(binfo);
+        g_free(binfo->book);
+	binfo->book = NULL;
         LOG(LOG_CRITICAL, "Failed to get the directory: %s", eb_error_message(error_code));
-        return NULL;
+        return False;;
     }
     binfo->subbook_dir = g_strdup(buff);
     error_code = eb_subbook_title2(binfo->book, sublist[binfo->subbook_no], buff);
     if(error_code != EB_SUCCESS)
     {
-        ebook_free(binfo);
+        g_free(binfo->book);
+	binfo->book = NULL;
         LOG(LOG_CRITICAL, "Failed to get the title: %s", eb_error_message(error_code));
-        return NULL;
+        return False;
     }
     binfo->title = iconv_convert(ENC_EUC_JP, ENC_UTF8, buff);
     if(!binfo->title)
     {
-        ebook_free(binfo);
-        return NULL;
+        g_free(binfo->book);
+	binfo->book = NULL;
+        return False;
     }
     error_code = eb_set_subbook(binfo->book, binfo->subbook_no);
     if(error_code != EB_SUCCESS)
     {
-        ebook_free(binfo);
+        g_free(binfo->book);
+	binfo->book = NULL;
         LOG(LOG_CRITICAL, "Failed to get the title: %s", eb_error_message(error_code));
-        return NULL;
+        return False;
     }
-    binfo->available = TRUE;
-
     if(eb_have_font(binfo->book, EB_FONT_16))
         eb_set_font(binfo->book, EB_FONT_16);
 
-    return binfo;
+    return True;
 }
 
 void ebook_free(BOOK_INFO *binfo)
@@ -135,10 +151,10 @@ void ebook_free(BOOK_INFO *binfo)
     g_free(binfo);
 }
 
-gint ebook_guess_gaiji_size(EB_Book *book, gint font_h)
+gint ebook_guess_gaiji_size(EB_Book *book)
 {
     gint size = EB_FONT_16;
-
+    gint font_h = mainwnd_get_font_size();
     if(font_h < 24)
         size = EB_FONT_16;
     else if(font_h < 30)
@@ -180,7 +196,7 @@ gchar *ebook_get_text(RESULT *res)
     text[len] = '\0';
     p = malloc(len + 1);
     g_strlcpy(p, text, len + 1);
-    return(p);
+    return p;
 }
 
 gchar *ebook_get_raw_text(EB_Book *book, gint page, gint offset)
@@ -273,11 +289,11 @@ GdkPixbuf* ebook_read_gaiji(BOOK_INFO *binfo, gchar *code, gint *w, gint *h)
     EB_Error_Code ec = EB_SUCCESS;
     GdkPixbuf *pixbuf;
 
-    if((size = ebook_guess_gaiji_size(book, headword.font_h)) < 0)
+    if((size = ebook_guess_gaiji_size(book)) < 0)
         return NULL;
-    ec = eb_set_font(book, EB_FONT_16);
+    ec = eb_set_font(book, size);
 
-    //memset(bitmap, 0, EB_SIZE_WIDE_FONT_48*sizeof(gchar));
+    memset(bitmap, 0, EB_SIZE_WIDE_FONT_48*sizeof(gchar));
 
     ec = eb_font_height(book, h);
     if(code[0] == 'h')
@@ -299,14 +315,24 @@ GdkPixbuf* ebook_read_gaiji(BOOK_INFO *binfo, gchar *code, gint *w, gint *h)
     return pixbuf;
 }
 
-static gint ebook_save_search_results(BOOK_INFO *binfo, const gchar *word, gint maxhits)
+static void ebook_search_results_filter(gpointer data, gpointer user_data)
 {
-    gint i, n, hits_count;
+    RESULT **res = (RESULT**)user_data;
+    if(result_compare((RESULT*)data, *res))
+    {
+	result_free(*res);
+	*res = NULL;
+    }
+}
+
+static gint ebook_search_results_save(BOOK_INFO *binfo, const gchar *word, gint maxhits)
+{
+    gint i, hits_count;
     gchar heading[MAX_BUFSIZE];
     ssize_t len;
     EB_Hit *hits = (EB_Hit*)g_new0(EB_Hit, maxhits);
     EB_Error_Code error_code;
-    RESULT *res, *prev = NULL;
+    RESULT *res;
 
     error_code = eb_hit_list(binfo->book, maxhits, hits, &hits_count);
     if(error_code != EB_SUCCESS)
@@ -314,40 +340,34 @@ static gint ebook_save_search_results(BOOK_INFO *binfo, const gchar *word, gint
         g_free(hits);
         return error_code;
     }
-    n = g_sequence_get_length(ebook.results);
-    for(i = 0; ((i + n) < maxhits) && (i < hits_count); i++)
+    for(i = 0; (i < maxhits) && (i < hits_count); i++)
     {
-        res = (RESULT*)g_new0(RESULT, 1);
-        if(!res)
-        {
-            LOG(LOG_ERROR, "No memory");
-            break;
-        }
-        error_code = eb_seek_text(binfo->book, &(hits[i].heading));
-        if(error_code != EB_SUCCESS)
-        {
-            g_free(res);
-            continue;
-        }
-        error_code = eb_read_heading(binfo->book, binfo->appendix, &hooksets.heading, NULL, MAX_BUFSIZE, heading, &len);
-        if(error_code != EB_SUCCESS)
-        {
-            g_free(res);
-            break;
-        }
-        heading[len] = '\0';
-
-        res->heading = iconv_convert(ENC_EUC_JP, ENC_UTF8, heading);
-
-        res->binfo = binfo;
-        res->pos = hits[i].text;
-	if(!result_compare(res, prev))
+	res = (RESULT*)g_new0(RESULT, 1);
+	if(!res)
 	{
-	    g_sequence_append(ebook.results, res);
-	    prev = res;
+	    LOG(LOG_ERROR, "No memory");
+	    break;
 	}
-	else
-	    result_free(res);
+	error_code = eb_seek_text(binfo->book, &(hits[i].heading));
+	if(error_code != EB_SUCCESS)
+	{
+	    g_free(res);
+	    continue;
+	}
+	error_code = eb_read_heading(binfo->book, binfo->appendix, &hooksets.heading, NULL, MAX_BUFSIZE, heading, &len);
+	if(error_code != EB_SUCCESS)
+	{
+	    g_free(res);
+	    break;
+	}
+	heading[len] = '\0';
+
+	res->heading = iconv_convert(ENC_EUC_JP, ENC_UTF8, heading);
+	res->binfo = binfo;
+	res->pos = hits[i].text;
+	g_sequence_foreach(ebook.results, ebook_search_results_filter, &res);
+	if(res)
+	    g_sequence_append(ebook.results, res);
     }
     g_free(hits);
     return error_code;
@@ -361,70 +381,85 @@ EB_Error_Code ebook_search_keyword(EB_Book *book, const char *input_word)
     return error_code;
 }
 
-void ebook_search_book(const gchar *word, gint method, GSequence *results, gint maxhits, BOOK_INFO *binfo)
+gboolean ebook_search_book(const gchar *word, gint method, GSequence *results, gint maxhits, BOOK_INFO *binfo)
 {
+    if(!binfo)
+	return False;
+    if(!binfo->book)
+	return False;
+    if(!methods[method].have_method(binfo->book))
+	return False;
+    gchar *euc_str = NULL;
+    if(word)
+    {
+	euc_str = iconv_convert(ENC_UTF8, ENC_EUC_JP, word);
+	if(!validate_euc(euc_str, False))
+	{
+	    g_free(euc_str);
+	    return False;
+	}
+    }
     if(results)
     {
         ebook_clear_results(results);
         ebook.results = results;
     }
-    if(methods[method].have_method == NULL)
-	return;
-    if(methods[method].search(binfo->book, word) == EB_SUCCESS)
-        ebook_save_search_results(binfo, word, maxhits);
+    if((method == SEARCH_METHOD_MENU) || (method == SEARCH_METHOD_COPYRIGHT))
+    {
+	EB_Position pos;
+	EB_Error_Code err;
+	gboolean menu = (method == SEARCH_METHOD_MENU);
+	if(menu)
+	    err = eb_menu(binfo->book, &pos);
+	else
+	    err = eb_copyright(binfo->book, &pos);
+	if(err == EB_SUCCESS)
+	{
+	    RESULT *res = (RESULT *)calloc(sizeof(RESULT), 1);
+	    res->heading = g_strdup_printf("%s : %s", menu ? _("menu") : _("copyright"), binfo->title);
+	    res->binfo = binfo;
+	    res->pos = pos;
+	    g_sequence_append(ebook.results, res);
+	}
+    }
+    else
+    {
+	if(methods[method].search(binfo->book, euc_str) == EB_SUCCESS)
+	{
+	    ebook_search_results_save(binfo, euc_str, maxhits);
+	}
+    }
+    g_free(euc_str);
+    return True;
 }
 
-void ebook_search(const gchar *word, gint method, GSequence *results, gint maxhits)
+gboolean ebook_search(const gchar *word, gint method, GSequence *results, gint maxhits, gboolean max_per_dict)
 {
     GtkTreeIter iter;
+    GtkTreeModel *store;
     BOOK_INFO *binfo;
-    if(!dictbar_get_group(&iter))
-        return;
-    ebook_clear_results(results);
-    ebook.results = results;
-    do {
-        gtk_tree_model_get(GTK_TREE_MODEL(dicts.store), &iter, DICT_BINFO, &binfo, -1);
-        if(!binfo->active)
-            continue;
-        ebook_search_book(word, method, NULL, maxhits, binfo);
-    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(dicts.store), &iter));
-    ebook.results = NULL;
-}
 
-void ebook_menu_or_copyright(gboolean menu, GSequence *results)
-{
-    GtkTreeIter   iter;
-    EB_Position pos;
-    RESULT *rp;
-    BOOK_INFO *binfo;
-    EB_Error_Code err;
-    ebook.results = results;
-
-    eb_textview_clear_textbuf(paned.view);
-    headword_clear();
     ebook_clear_results(results);
 
-    if(!dictbar_get_group(&iter))
-        return;
-    
+    if(!mainwnd_get_dicts(&store, &iter))
+        return False;
+
+    ebook.results = results;
+    gboolean ret = False;
     do {
-        gtk_tree_model_get(GTK_TREE_MODEL(dicts.store), &iter, DICT_BINFO, &binfo, -1);
+        gtk_tree_model_get(store, &iter, DICT_BINFO, &binfo, -1);
+	if(!binfo)
+	    continue;
         if(!binfo->active)
             continue;
-        if(menu ? !eb_have_menu(binfo->book) : !eb_have_copyright(binfo->book))
-            continue;
-        err = menu ? eb_menu(binfo->book, &pos) : eb_copyright(binfo->book, &pos);
-        if(err != EB_SUCCESS)
-            continue;
-        rp = (RESULT *)calloc(sizeof(RESULT), 1);
-        rp->heading = g_strdup_printf("%s : %s", menu ? _("menu") : _("copyright"), binfo->title);
-        rp->binfo = binfo;
-        rp->pos = pos;
-        g_sequence_append(ebook.results, rp);
-    } while(gtk_tree_model_iter_next(GTK_TREE_MODEL(dicts.store), &iter));
+        ret = ret | ebook_search_book(word, method, NULL, maxhits, binfo);
+	if(!max_per_dict)
+	    maxhits -= g_sequence_get_length(ebook.results);
+	if(maxhits <= 0)
+	    break;
+    } while (gtk_tree_model_iter_next(store, &iter));
     ebook.results = NULL;
-
-    headword_show_results();
+    return ret;
 }
 
 GdkPixbuf *ebook_load_image(BOOK_INFO *binfo, EB_Position *pos)
