@@ -2,18 +2,21 @@
 #ifndef __EBOOK_H_
 #define __EBOOK_H_
 
-gint            ebook_search_method();
-BOOK_INFO*      ebook_load(const gchar *path, int subbook_no);
+gboolean	ebook_has_method(EB_Book *book, gint method);
+const gchar*	ebook_index_to_method_name(gint index);
+
+gboolean	ebook_load(BOOK_INFO *binfo);
 GdkPixbuf*      ebook_load_image(BOOK_INFO *binfo, EB_Position *pos);
 void            ebook_free(BOOK_INFO *binfo);
 gchar*          ebook_get_text(RESULT *res);
 gchar*          ebook_get_raw_text(EB_Book *book, gint page, gint offset);
-void            ebook_search(const gchar *word, gint method, GSequence *results, gint maxhits);
-void            ebook_search_book(const gchar *word, gint method, GSequence *results, gint maxhits, BOOK_INFO *binfo);
+gboolean        ebook_search(const gchar *word, gint method, GSequence *results, gint maxhits, gboolean max_per_dict);
+gboolean        ebook_search_book(const gchar *word, gint method, GSequence *results, gint maxhits, BOOK_INFO *binfo);
 void            ebook_menu_or_copyright(gboolean menu, GSequence *results);
 GdkPixbuf*      ebook_read_gaiji(BOOK_INFO *binfo, gchar *code, gint *w, gint *h);
 EB_Error_Code	ebook_search_keyword(EB_Book *book, const char *input_word);
 
+
 enum
 {
     IMAGE_TYPE_JPEG, 
@@ -22,9 +25,4 @@ enum
     IMAGE_TYPE_GRAY_BMP 
 };
 
-struct _ebook
-{
-    GSequence   *results;
-} ebook;
-
-#endif
+#endif /* __EBOOK_H__ */
