@@ -2,7 +2,6 @@
 #include "defs.h"
 
 #include "jcode.h"
-#include "preferences.h"
 
 gchar *iconv_convert(const gchar *icode, const gchar *ocode, const gchar *orig)
 {
@@ -11,26 +10,6 @@ gchar *iconv_convert(const gchar *icode, const gchar *ocode, const gchar *orig)
     return g_convert(orig, strlen(orig), ocode, icode, &i, &o, &err);
 }
 
-gboolean validate_euc(gchar *str)
-{
-    gchar *p = str;
-    while(*p)
-    {
-        if(is_euc_printable((guchar*)p))
-            p +=2;
-        else if(isprint(*p))
-            p ++;
-        else if(isspace(*p))
-        {
-            *p = 0x20;
-            p++;
-        }
-        else
-            return FALSE;
-    }
-    return TRUE;
-}
-
 gboolean is_jis_printable(const guchar *buf)
 {
     g_assert(buf != NULL);
@@ -43,12 +22,43 @@ gboolean is_euc_printable(const guchar *buf)
     return ((0xA1 <= buf[0]) && (buf[0] <= 0xF4) && (0xA1 <= buf[1]) && (buf[1] <= 0xFE));
 }
 
-gchar *fs_to_unicode(gchar *from)
+gboolean validate_euc(gchar *str, gboolean jap_only)
 {
-    return iconv_convert(pref.codeset, ENC_UTF8, from);
-}   
-
-gchar *unicode_to_fs(gchar *from)
-{
-    return iconv_convert(ENC_UTF8, pref.codeset, from);
+    g_strstrip(str);
+    if(!strlen(str))
+	return False;
+    gchar *p = str;
+    while(*p)
+    {
+        if(is_euc_printable((guchar*)p))
+	{
+            p += 2;
+	    continue;
+	}
+	if(!jap_only)
+	{
+	    if(isprint(*p))
+	    {
+		p++;
+		continue;
+	    }
+	    else if(isspace(*p))
+	    {
+		*p = 0x20;
+		p++;
+		continue;
+	    }
+	}
+	return False;
+    }
+    return (p - str);
 }
+
+gboolean validate_utf8(const gchar *str, gboolean jap_only)
+{
+    gchar *euc_str = iconv_convert(ENC_UTF8, ENC_EUC_JP, str);
+    gboolean ret = validate_euc(euc_str, jap_only);
+    g_free(euc_str);
+    return ret;
+}
+
