@@ -2,13 +2,10 @@
 #ifndef __JCODE_H__
 #define __JCODE_H__
 
-#include "defs.h"
-
 gchar*      iconv_convert(const gchar *icode, const gchar *ocode, const gchar *orig);
-gboolean    validate_euc(gchar *str);
 gboolean    is_euc_printable(const guchar *buf);
 gboolean    is_jis_printable(const guchar *buf);
-gchar*      fs_to_unicode(gchar *from);
-gchar*      unicode_to_fs(gchar *from);
+gboolean    validate_euc(gchar *str, gboolean jap_only);
+gboolean    validate_utf8(const gchar *str, gboolean jap_only);
 
 #endif /* __JCODE_H__ */
